﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.IO;

using OWLIFT;

namespace OWLIFTReadCS
{
    public partial class Form1 : Form
    {
        private OwDev m_owDev;
        private byte[] m_decodeBuf;
        private Bitmap m_prevBmp;
        private Object m_lock = new Object();
        private bool m_setupTimeBaseAGC = false;
        private bool m_closing = false;
        private bool m_oneShotCap = false;
        private bool m_paused = false;
        private bool m_pausedBeforeTracking = false;
        private int m_lastCaptured;
        private UInt64 m_fileTimeLength;
        private static OwDecodeType LINEAR_AGC = OwDecodeType.LINEAR_AGC_RAINBOW_24;
        private static OwDecodeType NONLINEAR_AGC = OwDecodeType.NON_LINEAR_AGC_RAINBOW_24;
        private OwDecodeType m_decodeType = LINEAR_AGC;

        public Form1()
        {
            InitializeComponent();

            buttonOpen.Enabled = true;
            buttonPause.Enabled = false;
            buttonResume.Enabled = false;
            buttonReplay.Enabled = false;
            buttonM1.Enabled = false;
            buttonM10.Enabled = false;
            buttonM60.Enabled = false;
            buttonM600.Enabled = false;
            trackBar.Enabled = false;
            checkTAGC.Enabled = true;
            checkLinearAGC.Enabled = true;
            checkLinearAGC.Checked = true;

            Form1_SizeChanged(null, null);
        }

        private void Pause(bool withUI)
        {
            if (m_paused)
                return;

            if (withUI)
            {
                BeginInvoke((MethodInvoker)(() =>
                {
                    buttonPause.Enabled = false;
                    buttonResume.Enabled = true;
                }));
            }
            m_owDev.FileReadPause();
            m_paused = true;
        }

        private void Resume(bool withUI)
        {
            if (!m_paused)
                return;

            m_paused = false;

            if (withUI)
            {
                BeginInvoke((MethodInvoker)(() =>
                {
                    buttonPause.Enabled = true;
                    buttonResume.Enabled = false;
                }));
            }
            m_owDev.FileReadResume();
        }

        private void buttonOpen_Click(object sender, EventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.InitialDirectory = "C:\\";
            ofd.Filter = "OWI Files(*.owi)|*.owi|All Files(*.*)|*.*";
            ofd.FilterIndex = 1;

            if (ofd.ShowDialog() != DialogResult.OK)
            {
                return;
            }

            try
            {
                m_owDev = OwDev.FileReadSetup(ofd.FileName, CaptureProc, EndNotifyProc);
                m_decodeBuf = new byte[m_owDev.FrameWidth * m_owDev.FrameHeight * 3];
                m_prevBmp = new Bitmap(picBoxPreview.Width, picBoxPreview.Height);

                m_fileTimeLength = m_owDev.FileTimeLength;

                SetLabelTime(labelTimeLength, m_fileTimeLength);
                SetLabelDateTime(labelDateTime, m_owDev.FileHeadDateTime);
                trackBar.SetRange((int)m_owDev.FileHeadTimeOffset, (int)m_fileTimeLength);
                trackBar.SmallChange = 1000;
                trackBar.LargeChange = 10000;

                if (m_setupTimeBaseAGC)
                {
                    if (m_fileTimeLength < 300 * 1000)
                    {
                        throw new Exception("Too short file");
                    }
                    UInt32 intvl = 60;
                    UInt64 tms = 300 * 1000;
                    UInt64 tme = m_fileTimeLength - 60 * 1000;

                    m_owDev.FileTimeBasedAGCSetup(true, intvl, tms, tme);
                    m_owDev.FileTimeBasedAGCSetup(false, intvl, tms, tme);
                    m_owDev.TimeBasedAGC = true;
                }

                m_owDev.CaptureStart();
                buttonOpen.Enabled = false;
                buttonPause.Enabled = true;
                buttonResume.Enabled = false;
                buttonReplay.Enabled = true;
                buttonM1.Enabled = true;
                buttonM10.Enabled = true;
                buttonM60.Enabled = true;
                buttonM600.Enabled = true;
                checkTAGC.Enabled = false;
                trackBar.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK);
            }
        }

        private void Release()
        {
            if (m_owDev != null)
            {
                m_owDev.CaptureStop();
                m_owDev = null;
            }
        }

        private void buttonPause_Click(object sender, EventArgs e)
        {
            Pause(true);
        }

      　private void buttonResume_Click(object sender, EventArgs e)
        {
            Resume(true);
        }

        private void SetLabelTime(Label label, UInt64 time)
        {
            label.Text =
                (time / 1000 / 60 / 60).ToString("D") +
                ":" + ((time / 1000 / 60) % 60).ToString("D2") +
                ":" + ((time / 1000) % 60).ToString("D2");
        }

        private void SetLabelDateTime(Label label, OwDateTime dt)
        {
            label.Text =
                dt.year.ToString("D4") + "-" +
                dt.month.ToString("D2") + "-" +
                dt.day.ToString("D2") + " " +
                dt.hour.ToString("D2") + ":" +
                dt.minute.ToString("D2") + ":" +
                dt.second.ToString("D2") + "." +
                dt.millisecond.ToString("D3");
        }

        private unsafe void CaptureProc(byte[] buf)
        {
            int fx = m_owDev.FrameWidth, fy = m_owDev.FrameHeight;
            Bitmap bmp;
            int now = Environment.TickCount;
            UInt64 timePos = m_owDev.FileCurrentTimeOffset;

            if (m_lastCaptured == 0)
            {
                m_lastCaptured = now;
            }
            else if (m_fileTimeLength != timePos && now - m_lastCaptured < 33)
            {
                // Too short interval causes an error in picBoxPreview.
                return;
            }

            m_lastCaptured = now;

            try
            {
                m_owDev.Decode(buf, m_decodeBuf, m_decodeType);
            }
            catch (OwException)
            {
                return;
            }

            m_owDev.FinishDecode();

            fixed (void* p = m_decodeBuf)
            {
                bmp = new Bitmap(fx, fy, fx * 3,
                    PixelFormat.Format24bppRgb, new IntPtr(p));
            }

            try
            {
                lock (m_lock)
                {
                    Graphics g = Graphics.FromImage(m_prevBmp);

                    g.DrawImage(bmp, 0, 0, picBoxPreview.Width, picBoxPreview.Height);
                    picBoxPreview.Image = m_prevBmp;
                }
            }
            catch (Exception)
            {
                return;
            }

            if (!m_closing &&
                !labelTimePos.IsDisposed && !labelDateTime.IsDisposed &&
                !trackBar.IsDisposed && !m_paused)
            {
                BeginInvoke((MethodInvoker)(() =>
                {
                    SetLabelTime(labelTimePos, timePos);
                    SetLabelDateTime(labelDateTime, m_owDev.FileCurrentLocalDateTime);
                    trackBar.Value = (int)timePos;
                }));
            }

            if (m_oneShotCap)
            {
                if (m_pausedBeforeTracking)
                    Pause(false);
                m_oneShotCap = false;
            }
        }

        private unsafe void EndNotifyProc()
        {

        }

        private void buttonExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            m_closing = true;
            Release();
        }

        private void buttonReplay_Click(object sender, EventArgs e)
        {
            m_owDev.FileReadSeek(0);
            Resume(true);
        }

        private void trackBar_Scroll(object sender, EventArgs e)
        {
            try
            {
                m_owDev.FileReadSeek((ulong)trackBar.Value);
                m_oneShotCap = true;
                Resume(false);
            }
            catch (OwException)
            {
            }
        }

        private void trackBar_MouseDown(object sender, MouseEventArgs e)
        {
            m_pausedBeforeTracking = m_paused;
        }

        private void trackBar_MouseUp(object sender, MouseEventArgs e)
        {
        }

        private void SetSpeed(double speed)
        {
            if (!m_paused)
                m_owDev.FileReadPause();
            m_owDev.FileReadSpeed = speed;
            if (!m_paused)
                m_owDev.FileReadResume();
        }

        private void buttonM1_Click(object sender, EventArgs e)
        {
            SetSpeed(1);
        }

        private void buttonM10_Click(object sender, EventArgs e)
        {
            SetSpeed(10);
        }

        private void buttonM60_Click(object sender, EventArgs e)
        {
            SetSpeed(60);
        }

        private void buttonM600_Click(object sender, EventArgs e)
        {
            SetSpeed(600);
        }

        private void Form1_SizeChanged(object sender, EventArgs e)
        {
            lock (m_lock)
            {
                Rectangle rForm = this.ClientRectangle;
                Rectangle rPixBox = picBoxPreview.Bounds;
                Rectangle rTimePos = labelTimePos.Bounds;
                Rectangle rTimeLen = labelTimeLength.Bounds;
                Rectangle rDateTime = labelDateTime.Bounds;
                Rectangle rTrackBar = trackBar.Bounds;

                if (rForm.Width == 0)
                    return;

                rPixBox.Width = rForm.Width - 154;
                rPixBox.Height = rForm.Height - 74;
                rTimePos.Y = rPixBox.Top + rPixBox.Height + 4;
                rDateTime.Y = rTimePos.Y;
                rTimeLen.X = rPixBox.Right - rTimeLen.Width;
                rTimeLen.Y = rTimePos.Y;
                rTrackBar.Width = rForm.Width - 16;
                rTrackBar.Y = rPixBox.Bottom + 30;

                picBoxPreview.Bounds = rPixBox;
                labelTimePos.Bounds = rTimePos;
                labelTimeLength.Bounds = rTimeLen;
                labelDateTime.Bounds = rDateTime;
                trackBar.Bounds = rTrackBar;

                m_prevBmp = new Bitmap(rPixBox.Width, rPixBox.Height);
            }
        }

        private void checkTAGC_CheckedChanged(object sender, EventArgs e)
        {
            if (m_owDev == null)
                m_setupTimeBaseAGC = checkTAGC.Checked;
            else
                m_owDev.TimeBasedAGC = checkTAGC.Checked;
        }

        private void checkLinearAGC_CheckedChanged(object sender, EventArgs e)
        {
            m_decodeType = checkLinearAGC.Checked ? LINEAR_AGC : NONLINEAR_AGC;
        }
    }
}
